
disp('===============> Getting parameters, mex-c code, and training images');
mkdir('working'); % working directory to store .mat files. 
mkdir('output'); % directory that contains output from the code. 

%% parameters for the outer square bounding box
category = 'TigerFace';
outerBBSize = 144; % the size of the bounding square box which holds the template lattice
templateSize = floor(.5 + [outerBBSize outerBBSize]); % object template size
%% parameters for Gabor filters
numScale = 1;
scales = [1]; % scales of Gabor wavelets
numOrient = 16;  % number of orientations
saturation = 6.; % saturation level for sigmoid transformation
%% parameters for exponential model
binSize = .2;  % binsize for computing histogram of q()
numStoredPoint = 50; % number of stored lambda values
spacing = .1; % spacing between two adjacent points
%% parameters for normalization
localOrNot = 1; % if we use local normalization or not. If not, set it to -1 
localHalfx1 = 20; localHalfy1 = 20; % the half range for local normalization, has to be quite large
windowNormalizeOrNot = -1; % whether normalize within the scanning window in detection 
if (localOrNot>0)
    windowNormalizeOrNot = -1;
end % if we use local normalization, we should not use window normalization in detection
thresholdFactor = .01;  % divide the response by max(average, maxAverage*thresholdFactor)
%% parameters for active basis
epsilon = .1; % allowed correlation between selected Gabors 
subsample = 1; % subsample in computing MAX1 maps
locationShiftLimit = 3; % shift in normal direction = locationShiftLimit*subsample pixels
orientShiftLimit = 1; % shift in orientation
numElement1 = 20; % number of Gabors in active basis at the first scale
numIt = 5; % number of EM-type iterations (alternative Find and Learn)
%% read in positive images
sizeTemplatex = templateSize(1);
sizeTemplatey = templateSize(2);
halfTemplatex = floor(sizeTemplatex/2);
halfTemplatey = floor(sizeTemplatey/2);

LambdaThreshold = .4;

Iname = dir('positiveImage/*.jpg');
numImage = length(Iname);

%% part locations and deformations
partSize = outerBBSize/2;
PartLocX = 1:partSize/4:templateSize(1)-partSize+1;
PartLocY = 1:partSize/4:templateSize(2)-partSize+1;
partLocRange = floor(partSize/6);
partInhibitionMethod = 'boundingbox'; % the method of local inhibition for adjacent partial templates 
partGainThreshold = 20; % the selection threshold of partial templates to compose into an Active Plate 
zipfilename = sprintf('AP_WithFind_%s_20101129.zip',category);
save('Config.mat','partSize','PartLocX','PartLocY','partLocRange','starting',...
    'category','outerBBSize','templateSize','numOrient',...
    'numElement1','locationShiftLimit','orientShiftLimit','category','numIt',...
    'zipfilename','numImage','LambdaThreshold',...
    'partInhibitionMethod','partGainThreshold');


