outputFolder = 'Lioness';
category = outputFolder;

%% Load in exponential model, mex C codes, and set parameters
load 'storedExponentialModel'; % load in exponential model

locationPerturbFraction = 1/10; % part perturbation
sizeTemplatex = 240; sizeTemplatey = sizeTemplatex; %126
templateSize = [sizeTemplatex sizeTemplatex];
epsilon = .1; % allowed correlation between selected Gabors 
subsample = 1; % subsample in computing MAX1 and SUM2 maps   
subsampleS2 = 1; subsampleM2 = 1;
locationShiftLimit = 3; % shift in normal direction = locationShiftLimit*subsample pixels
orientShiftLimit = 1; % shift in orientation
Correlation = CorrFilter(allFilter, epsilon); % correlation between filters 
numElement = 130; % number of Gabors in active basis
numIteration = 5;  % number of iterations
% LambdaThreshold = .6;

partRotationRange = 1*(-2:2); % part rotation (rotation of partial templates)
numPartRotate = length(partRotationRange);

minRotationDif = (sin(1*pi/numOrient)-sin(0))^2 + (cos(1*pi/numOrient)-cos(0))^2 + 1e-10;
% minRotationDif = 1e-10;
rotationRange = 1*(-1:1); % whole object rotation
numRotate = length(rotationRange);

% bounding box of the first object
startx = 3; endx = startx + sizeTemplatex - 1; % initialize by learning from a given patch
starty = 6; endy = starty + sizeTemplatey - 1;

resizeFactor = 1.; % resize the input images  0.5
numResolution = 5;  % number of resolutions to search for in detection stage
originalResolution = 3; % original resolution is the one at which the imresize factor = 1, see 11th line beneath this line 
starting = 1; % initilize by single image learning from I{starting}

partSize = sizeTemplatex/4;
partMargin = floor(partSize/4);
PartLocX0 = 1:partSize:templateSize(1)-partSize+1;
PartLocY0 = 1:partSize:templateSize(2)-partSize+1;
partLocRange = floor(partSize*locationPerturbFraction);
numCandPart = length(PartLocX0) * length(PartLocY0);
PartLocX = zeros(numCandPart,1);
PartLocY = zeros(numCandPart,1);
iPart = 1;
for x = PartLocX0
    for y = PartLocY0
        PartLocX(iPart) = x;
        PartLocY(iPart) = y;
        iPart = iPart + 1;
    end
end

partGainThreshold = 20; % the selection threshold of partial templates to compose into an Active Plate 
zipfilename = sprintf('HAPMorph_%s_20110111.zip',category);

imageFolder = 'positiveImage'; % folder of training images  
imageName = dir([imageFolder '/*.jpg']);
numImage = size(imageName, 1); % number of training images 

save Config

%%

%% Load in training images and initialize SUM1 maps for learning
I = cell(1, numImage);
for img = 1 : numImage
	tmpIm = imread([imageFolder '/' imageName(img).name]); 
	if size(tmpIm,3) == 3
		tmpIm = rgb2gray(tmpIm);
	end
	I{img} = imresize(single(tmpIm), resizeFactor, 'nearest');
end
